// Copyright 2005 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.hivemind.internal.ser;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hivemind.ApplicationRuntimeException;

/**
 * Class used to hold a <em>global</em> instance of
 * {@link org.apache.hivemind.internal.ser.ServiceSerializationSupport}, so that
 * {@link org.apache.hivemind.internal.ser.ServiceToken}s may locate them.
 * 
 * @author Howard M. Lewis Ship
 * @since 1.1
 */
public class ServiceSerializationHelper
{
    private static final Log LOG = LogFactory.getLog(ServiceSerializationHelper.class);

    private static ServiceSerializationSupport _serviceSerializationSupport;

    /**
     * Returns the previously stored SSS.
     * 
     * @throws ApplicationRuntimeException
     *             if no SSS has been stored.
     */
    public static ServiceSerializationSupport getServiceSerializationSupport()
    {
        if (_serviceSerializationSupport == null)
            throw new ApplicationRuntimeException(SerMessages.noSupportSet());

        return _serviceSerializationSupport;
    }

    /**
     * Stores the SSS instance for later access; if an existing SSS is already stored, then an error
     * is logged (should be just one SSS per class loader).
     */

    public static void setServiceSerializationSupport(
            ServiceSerializationSupport serviceSerializationSupport)
    {
        if (serviceSerializationSupport != null && _serviceSerializationSupport != null)
            LOG.error(SerMessages.supportAlreadySet(
                    serviceSerializationSupport,
                    _serviceSerializationSupport));

        _serviceSerializationSupport = serviceSerializationSupport;
    }

    /**
     * Invoked to clear the SSS, if it matches the provide parameter. This is necessary, rather than
     * blindly setting it to null, because of a tricky case related to
     * {@link org.apache.hivemind.servlet.HiveMindFilter}, which includes the ability to create a
     * new Registry on the fly and has to leave both Registry's running for a brief window.
     */

    public static void resetServiceSerializationSupport(
            ServiceSerializationSupport serviceSerializationSupport)
    {
        if (_serviceSerializationSupport == serviceSerializationSupport)
            _serviceSerializationSupport = null;
    }

}